import bpy
import time
from scalerender.utils.util import ScaleRenderUploadManager

class RENDERFARM_OT_RenderOnFarm(bpy.types.Operator):
    """Operator which sends the current .blend file to ScaleRender farm."""
    bl_idname = "scalerender.render_operator"
    bl_label = "Start Render on ScaleRender"
    bl_options = {'REGISTER', 'UNDO'}

    _timer = None
    managers = {}  # Dictionary to hold multiple upload managers

    render_name: bpy.props.StringProperty(
        name="Render Name",
        description="Enter a name for your render",
        default="MyRender"
    )

    def get_user_token(self):
        """Retrieve the user token from preferences."""
        return getattr(bpy.context.preferences.addons["scalerender"].preferences, 'scalerender_token', "")

    def modal(self, context, event):
        if event.type == 'TIMER':
            for unique_id, manager in list(self.managers.items()):
                if manager.is_upload_done():
                    if manager.status == 'DONE':
                        self.report({'INFO'}, f"Successfully sent {manager.render_name} to ScaleRender!")
                    else:
                        self.report({'ERROR'}, f"Failed to send {manager.render_name} to ScaleRender: {manager.reason}")
                    self.managers.pop(unique_id)

            if not self.managers:
                self.cancel(context)
                return {'FINISHED'}

        return {'PASS_THROUGH'}

    def execute(self, context):
        """Start the upload process."""
        user_token = self.get_user_token()
        if not user_token:
            self.report({'ERROR'}, "You must be logged in to ScaleRender to render. Please log in and try again.")
            return {'CANCELLED'}

        file_name = f"{self.render_name}.blend"
        blender_version = '4.0.1' if bpy.app.version >= (4, 0, 0) else '3.4.0'
        settings = context.scene.scalerender_settings

        # Create a unique identifier using name and timestamp
        unique_id = f"{self.render_name}_{int(time.time())}"
        manager = ScaleRenderUploadManager(unique_id, user_token, file_name, settings, blender_version)
        self.managers[unique_id] = manager  # Store the manager with the unique ID
        manager.start_upload()

        self.report({'INFO'}, f"Uploading {unique_id} to scalerender.net")

        if not self._timer:
            self._timer = context.window_manager.event_timer_add(10.0, window=context.window)
            context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def cancel(self, context):
        """Cancel the render operation."""
        if self._timer:
            context.window_manager.event_timer_remove(self._timer)
            self._timer = None

    def invoke(self, context, event):
        """Invoke the render operator with a dialog to ask for the render name."""
        return context.window_manager.invoke_props_dialog(self)

    def draw(self, context):
        """Draw the dialog for user input."""
        self.layout.prop(self, "render_name")

def menu_func(self, context):
    """Add the operator to the render menu."""
    self.layout.operator(RENDERFARM_OT_RenderOnFarm.bl_idname, text="Start Render on ScaleRender")

def register():
    """Register the operator and add it to the menu."""
    bpy.utils.register_class(RENDERFARM_OT_RenderOnFarm)
    bpy.types.TOPBAR_MT_render.append(menu_func)

def unregister():
    """Unregister the operator and remove it from the menu."""
    bpy.utils.unregister_class(RENDERFARM_OT_RenderOnFarm)
    bpy.types.TOPBAR_MT_render.remove(menu_func)