import bpy
from bpy.props import StringProperty, IntProperty, CollectionProperty, BoolProperty
from bpy.types import PropertyGroup, Operator
from ..utils import util  # Adjust this import path as needed based on your package structure

# Define a PropertyGroup for render statuses
class RenderStatusItem(PropertyGroup):
    name: StringProperty(name="Render Name")
    pct_done: StringProperty(name="Percent Done")
    download_link: StringProperty(name="Download Link")


# Define an operator to fetch and display render status
class RENDERFARM_OT_ScaleRenderRenders(Operator):
    """Statuses of current renders"""
    bl_idname = "scalerender.render_statuses"
    bl_label = "ScaleRender Job Statuses"
    bl_options = {'REGISTER', 'UNDO'}

    out_of_funds: BoolProperty()
    render_statuses: CollectionProperty(type=RenderStatusItem)

    def execute(self, context):
        # Data has already been fetched by invoke()
        return {'FINISHED'}

    def invoke(self, context, event):
        # Fetch render statuses when the operator is invoked
        self.fetch_render_statuses(context)
        return context.window_manager.invoke_props_dialog(self, width=400)


    def draw(self, context):
        layout = self.layout

        # Out of funds warning
        if self.out_of_funds:
            layout.label(text="Out of funds! Add more at https://scalerender.net/add_funds_amount", icon='ERROR')

        # No renders message
        if not self.render_statuses:
            layout.label(text="You don't currently have any recently started renders.")
        else:
            # Create the table headers
            row = layout.row()
            row.label(text="Render Name")
            row.label(text="Progress")
            row.label(text="Download Link")

            # Create a row for each render status
            for render_status in self.render_statuses:
                row = layout.row(align=True)  # Set the align attribute to True
                row.label(text=f"{render_status.name}")
                row.label(text=f"{render_status.pct_done}% done")

                if render_status.download_link:
                    op = row.operator("wm.url_open", text="Download")
                    op.url = render_status.download_link
                else:
                    # Add an empty operator or label to maintain consistent alignment
                    row.label(text="")

        # Link to rendered files as text (not as a button)
        row = layout.row()
        row.label(text='See https://scalerender.net/renderedFiles for more details')

    def fetch_render_statuses(self, context):
        token = context.preferences.addons["scalerender"].preferences.scalerender_token
        response, status, reason = util.make_request('GET', "/current_render_statuses/", token=token)

        if status == 200:
            self.out_of_funds = response.get('out_of_funds', False)
            self.render_statuses.clear()

            for render in response.get('statuses', []):
                item = self.render_statuses.add()
                item.name = render.get('render_name')
                item.pct_done = render.get('pct_done')
                item.download_link = render.get('download_link', '')
        else:
            self.report({'ERROR'}, f"Failed to fetch render statuses: {reason}")
            self.out_of_funds = False
            self.render_statuses.clear()


def menu_func(self, context):
    self.layout.operator(RENDERFARM_OT_ScaleRenderRenders.bl_idname, text="Render Statuses")


def register():
    bpy.utils.register_class(RenderStatusItem)
    bpy.utils.register_class(RENDERFARM_OT_ScaleRenderRenders)
    bpy.types.TOPBAR_MT_render.append(menu_func)


def unregister():
    bpy.utils.unregister_class(RenderStatusItem)
    bpy.utils.unregister_class(RENDERFARM_OT_ScaleRenderRenders)
    bpy.types.TOPBAR_MT_render.remove(menu_func)