import bpy
import json
import os
import time
import uuid
import tempfile
import urllib.request
import threading

# BASE_URL = "http://localhost:8000"
BASE_URL = "https://scalerender.net"


def make_request(method, path, data=None, token=None, headers=None):
    """Utility function to make a request using urllib."""
    url = f"{BASE_URL}{path}"
    req_headers = {}
    if token:
        req_headers["Authorization"] = f"Token {token}"
    if headers:
        req_headers.update(headers)

    request = urllib.request.Request(url, data=data, headers=req_headers, method=method)
    print("sending to: [" + str(url) + "] with headers [" + str(req_headers) + "] and method [" + str(method) + "]")
    with urllib.request.urlopen(request) as response:
        content_type = response.headers.get('Content-Type', '')

        # If the response is JSON
        if 'application/json' in content_type:
            body = response.read().decode('utf-8')
            print("received: [" + str(body) + "]")
            return json.loads(body), response.status, response.reason
        else:
            # If it's binary data, just return raw bytes
            body = response.read()
            return body, response.status, response.reason


def upload_blend_to_server(token, file_name, priority, engine, max_memory, blender_version='4.0.1'):
    # Use the system's temporary directory
    temp_dir = tempfile.gettempdir()

    bpy.ops.file.pack_all()
    file_path = os.path.join(temp_dir, file_name)
    bpy.ops.wm.save_as_mainfile(filepath=file_path)

    # Create a multipart/form-data POST request
    boundary = 'wL36Yn8afVp8Ag7AmP8qZ0SA4n0v9T'
    headers = {
        "Content-Type": f"multipart/form-data; boundary={boundary}",
        "Authorization": f"Token {token}",
        "Accept": "application/json",
        "Content-Disposition": f"attachment; filename={file_name}",
        "X-Priority": priority,
        "X-Engine": engine,
        "X-Maxmem": max_memory,
        "X-Blenderversion": blender_version
    }

    # Build the multipart body
    with open(file_path, "rb") as f:
        file_data = f.read()

    json, status, reason = make_request('POST', "/plugInRenderFileUpload/", data=file_data, headers=headers)

    render_uuid = json.get('render_uuid', None)
    os.remove(file_path)
    return status, reason, render_uuid


class ScaleRenderUploadManager:
    def __init__(self, render_name, user_token, file_name, settings, blender_version):
        self.render_name = render_name
        self.user_token = user_token
        self.file_name = file_name
        self.settings = settings
        self.blender_version = blender_version
        self.status = 'PENDING'
        self.reason = ''
        self.upload_thread = None

    def start_upload(self):
        """Starts the upload in a separate thread."""
        self.upload_thread = threading.Thread(target=self.upload)
        self.upload_thread.start()

    def upload(self):
        """Handle the actual upload process."""
        try:
            status, reason, render_uuid = upload_blend_to_server(
                token=self.user_token,
                file_name=self.file_name,
                priority=self.settings.priority,
                engine=self.settings.engine,
                max_memory=self.settings.max_memory,
                blender_version=self.blender_version
            )
            if status == 200:
                self.status = 'DONE'
            else:
                self.status = 'FAILED'
                self.reason = reason
        except Exception as e:
            self.status = 'FAILED'
            self.reason = str(e)

    def is_upload_done(self):
        """Check if the upload is done."""
        if self.upload_thread and not self.upload_thread.is_alive():
            return True
        return False